--
--
-- FPGA Life VHDL Module - Conway's Game of Life for Altera University Board 
-- Uses Flex 10K20 Device
--
-- Jim Hamblen, Georgia Tech School of ECE
--
-- PB1 is run/stop (powers up in stop mode allows monitor to sync - problems starting after stop!)
-- PB2 is single step (in stop mode - not working now!) 
-- Flex Switch 7..4 is Speed (try sw7,6,5,4 closed for fast update)
-- Flex Switch 2 is Reverse Video
-- Flex Switch 3 selects different initial life pattern
--
library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;


entity flexlife is

Generic(ADDR_WIDTH: integer := 12; DATA_WIDTH: integer := 1);

   port( signal PB1, PB2, Clock : in std_logic;
        signal LSB_a, LSB_b, LSB_c, LSB_d, LSB_e, LSB_f, LSB_g, LSB_dp,
        MSB_a, MSB_b, MSB_c, MSB_d, MSB_e, MSB_f, MSB_g, MSB_dp : out std_logic;               
        signal Red,Green,Blue : out std_logic;
        signal Horiz_sync,Vert_sync : out std_logic;
        signal MOUSE_DATA: in std_logic;
        signal MOUSE_CLK : in std_logic;
        signal TEMP_MEM_OUTPUT, TEMP_PB1_SINGLE_PULSE, TEMP_SP : out std_logic;
        signal Flex_Switch_1, Flex_Switch_2, Flex_Switch_3, Flex_Switch_4: in std_logic;
        signal Flex_Switch_5, Flex_Switch_6, Flex_Switch_7, Flex_Switch_8: in std_logic);
       
		
		
end flexlife;

architecture behavior of flexlife is

-- Life Game Signals
signal game_address, init_game_address: std_logic_vector(11 Downto 0);
signal generation_count: std_logic_vector(7 Downto 0);
signal game_col_address, game_row_address: std_logic_vector(5 Downto 0); 
signal temp_address: std_logic_vector(6 Downto 0);
signal neighbor_col_adjust, neighbor_row_adjust: std_logic_vector(5 Downto 0);
signal neighbor_count_state: std_logic_vector (3 Downto 0);
signal nearest_neighbor_count: std_logic_vector(3 Downto 0);
signal Game_Clock, Game_compute_clock, Game_memory_address, Game_memory_we : std_logic; 
signal active_page, game_run, single_gen_compute, speed_signal: std_logic; 
signal power_on, half_speed_clock: std_logic;
signal speed: std_logic_vector(3 Downto 0);
signal switch, switch_sync: std_logic_vector(7 Downto 0);

-- Video Display Signals   
signal video_address: std_logic_vector(11 Downto 0);
signal H_count,V_count: std_logic_vector(9 Downto 0);
signal F_count: std_logic_vector(4 Downto 0);
signal Color_count: std_logic_vector(3 Downto 0);
signal Red_Data, Green_Data, Blue_Data: std_logic;

-- Signals for Video Memory for Pixel Data
signal address_a, address_b: std_logic_vector(11 Downto 0);
signal data, data_a, data_b, mem_output, mem_output_a, mem_output_b: std_logic_vector(0 Downto 0);
signal we, we_a, we_b, inclock: std_logic;
signal col_address, row_address: std_logic_vector(5 Downto 0);
signal pixel_col_count, pixel_row_count: std_logic_vector(5 Downto 0);

-- Signals for LED Display
signal LSB,MSB: std_logic_vector(3 Downto 0);
signal LSB_7SEG,MSB_7SEG: std_logic_vector(6 Downto 0);

-- Signals for Push buttons
signal PB1_sync, PB2_sync, PB2_Single_Pulse, PB1_Single_Pulse: std_logic; 
signal PB2_debounced, PB1_debounced, PB2_debounced_Sync, PB1_debounced_Sync: std_logic; 
signal PB1_debounced_delay, PB2_debounced_delay, Debounce_clock: std_logic;
signal SHIFT_PB1, SHIFT_PB2: std_logic_vector(3 Downto 0);

-- Signals for Mouse
signal CHARIN: std_logic_vector(7 downto 0);
signal DELTA_Y, DELTA_X : std_logic_vector(15 downto 0); 
signal Y_CURSOR, X_CURSOR : std_logic_vector(15 downto 0);
signal INCNT : std_logic_vector(3 Downto 0);
signal PACKET_COUNT : std_logic_vector(1 Downto 0);
signal SHIFTIN : std_logic_vector(8 Downto 0);
signal PACKET_CHAR1, PACKET_CHAR2, PACKET_CHAR3 : std_logic_vector(7 downto 0); 
signal BUF_MOUSE_CLK, DATA_READY, READ_CHAR, LEFT_BUTTON, RIGHT_BUTTON, RESET: std_logic;
signal i: integer;
constant H_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(799,10); 
-- 799 is max horiz count
constant V_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(524,10); 
-- 524 is max vert count
signal video_on, video_on_H, video_on_V: std_logic;
begin           
------------------------------------------------------------------------------
-- Two Banks of 64 by 64 by 1 video ram for pixel data
-- Signal active_page selects bank to update data and bank to display data
-- Banks Ping Pong Between update data and display function
-- row address is first six bits and column address is last six bits
 video_ram_a: lpm_ram_dq
      GENERIC MAP (lpm_widthad => ADDR_WIDTH,
        lpm_outdata => "UNREGISTERED",
        lpm_indata => "UNREGISTERED",
        lpm_address_control => "UNREGISTERED",
-- Reads in mif file for initial data - a Spaceship (periodic moving pattern)
         lpm_file => "starship.mif",
         lpm_width => DATA_WIDTH)
      PORT MAP (data => data_a, address => address_a, we => we_a, q => mem_output_a);
 
video_ram_b: lpm_ram_dq
      GENERIC MAP (lpm_widthad => ADDR_WIDTH,
        lpm_outdata => "UNREGISTERED",
        lpm_indata => "UNREGISTERED",
        lpm_address_control => "UNREGISTERED",
-- Reads in mif file for initial data - a Rabbit Methuselah (simple long lived pattern)
         lpm_file => "rabbit.mif",
         lpm_width => DATA_WIDTH)
      PORT MAP (data => data_b, address => address_b, we => we_b, q => mem_output_b);

-- Video unit drives address when pixel data needed otherwise game compute unit
address_a <= game_address WHEN game_memory_address = '1' ELSE video_address;
address_b <= game_address WHEN game_memory_address = '1' ELSE video_address;

-- Write new data to update active compute bank 
we_a <= we WHEN active_page = '0' ELSE '0';
we_b <= we WHEN active_page = '1' ELSE '0';
data_b <= data;
data_a <= data;
inclock <= clock;
mem_output <= mem_output_a WHEN active_page='1' ELSE mem_output_b;
temp_mem_output <= mem_output(0);
temp_sp <= Single_gen_compute;


-- Colors for pixel data on video signal
-- address video_ram for pixel color data
-- Switch 2 will reverse video
Red_Data <= not (mem_output(0) xor Switch_Sync(2));
Green_Data <= not (mem_output(0) xor Switch_Sync(2));
Blue_Data <= '1';

video_address(11 Downto 6) <= row_address;
video_address(5 Downto 0) <= col_address;

Red <=   Red_Data and video_on;
Green <= Green_Data and video_on;
Blue <=  Blue_Data and video_on;

-- video_on turns off pixel data when not in the view area
video_on <= video_on_H and video_on_V;

-- Update Pixel Data in RAM for Next Generation of Life
    game_address(11 downto 6) <= game_row_address + neighbor_row_adjust;
    temp_address(6 downto 0) <= game_col_address + neighbor_col_adjust;
    game_address(5 downto 0) <= temp_address(5 downto 0) ;

LIFE_GAME_COMPUTE: Process (clock)
Begin
 Wait until (clock'event) and (clock='1');
-- Flag for Single cycle step operation
IF PB2_SINGLE_PULSE ='1' THEN Single_Gen_Compute <='1'; End if;
-- Power_On is set to zero at power up so this is used for reset
 If Power_On ='0' Then
    we <='0';
-- Choose Initial Life Pattern with Switch 3
    active_page <= Switch(3);
    neighbor_count_state <= "0000";
    nearest_neighbor_count <= "0000";
    generation_count <= "00000000";
    game_col_address <= "000000";
    game_row_address <= "000000";
    Game_memory_we <= '0';
    Game_memory_address <= '0';
    Power_On <= '1';
  Else

-- Used to slow Down Video Memory Timing Needs 2*40ns Cycles
Half_Speed_Clock <= Not Half_Speed_Clock;


-- Game memory address signals go active when memory not needed for video display
-- Switch address mux one clock cycle prior to video use because of
-- tight (80ns) timing for memory read

If (H_count >= 797) OR (H_count <= 640) Then Game_memory_address <='0';
 ELSE Game_memory_address <='1'; End if;


IF (H_Count >= CONV_STD_LOGIC_VECTOR(642,10) AND 
   H_Count<= CONV_STD_LOGIC_VECTOR(796,10)) 
Then
-- Check for speed control, single step and run states
 IF (((Game_Run = '1' AND Speed_Signal = '1' ) OR (Single_Gen_Compute ='1')) 
      AND (Half_Speed_Clock='1') )
 Then
-- This state machine counts the number of live 8 nearest neighbor pixels in 
-- the display bank and creates the next generation of pixel data in the other 
-- memory bank. When a new generation of pixels is ready the banks are switched 
-- and the process repeats
CASE neighbor_count_state IS
-- Compute Number of Live Neighbors
        WHEN "0000" =>
            we <='0';
-- Compute Address for North Neighbor (used in next state for read cycle)
-- Note 111..1 is -1 in two's complement
            neighbor_row_adjust <= "111111"; 
            neighbor_col_adjust <= "000000";
            neighbor_count_state <= "0001";
       WHEN "0001" =>
            neighbor_row_adjust <= "111111"; 
            neighbor_col_adjust <= "000001";
            neighbor_count_state <= "0010";
-- North Neighbor
            If mem_output /= "0"  Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
       WHEN "0010" =>
            neighbor_row_adjust <= "000000"; 
            neighbor_col_adjust <= "000001";
            neighbor_count_state <= "0011";
-- Northeast Neighbor
            If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
      WHEN "0011" =>
            neighbor_row_adjust <= "000001"; 
            neighbor_col_adjust <= "000001";
            neighbor_count_state <= "0100";
 -- East Neighbor
           If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
       WHEN "0100" =>
            neighbor_row_adjust <= "000001"; 
            neighbor_col_adjust <= "000000";
            neighbor_count_state <= "0101";
 -- Southeast Neighbor
           If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
      WHEN "0101" =>
            neighbor_row_adjust <= "000001"; 
            neighbor_col_adjust <= "111111";
            neighbor_count_state <= "0110";
-- South Neighbor
            If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
        WHEN "0110" =>
            neighbor_row_adjust <= "000000"; 
            neighbor_col_adjust <= "111111";
            neighbor_count_state <= "0111";
-- Southwest Neighbor
            If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
       WHEN "0111" =>
            neighbor_row_adjust <= "111111"; 
            neighbor_col_adjust <= "111111";
            neighbor_count_state <= "1000";
-- West Neighbor
            If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
-- Compute new pixel data for memory        
	   WHEN "1000" =>
            neighbor_row_adjust <= "000000"; 
            neighbor_col_adjust <= "000000";
            neighbor_count_state <= "1001";
-- Northwest Neighbor
           If mem_output /= "0" Then
             Nearest_Neighbor_Count <= Nearest_Neighbor_Count + 1;
            end if;
	   WHEN "1001" =>
            neighbor_count_state <= "1010";
            we <= '0';
-- Check Conway's Game of Life Rules to write new pixel data for next generation
-- Survival if 2 Live Neighors 
            IF Nearest_Neighbor_count ="0010" Then 
            Data <= mem_output ;
            ELSE
-- Birth  if 3 Live Neighbors  
-- Write New Live Cell
            IF Nearest_neighbor_count ="0011" Then Data <= "1";
            ELSE
-- Death   if <2 or >3 Live Neighbors
-- Write Dead Cell
            Data <= "0";
            End if;
            End if;
         WHEN "1010" =>
-- Write new Pixel_Data
-- Wait if not enough time to finish write cycle before display data needed
	-- Note: Time needed is a function of switch speed settings
         If H_count >= (CONV_STD_LOGIC_VECTOR(790,10) - (Switch_Sync(7 Downto 4) & "00")) Then neighbor_count_state <="1010";
         ELSE neighbor_count_state <= "1011";
         End if;
           we <= '0'; 
--           inclock <= '0';
        WHEN "1011" =>
--           inclock <= '1';
           we <='1';
           neighbor_count_state <= "1100";
        WHEN "1100" =>
--           inclock <= '1';
           we <='0';
           neighbor_count_state <= "1101";

--Move to Next Pixel
         WHEN "1101" =>
           we <='0';
  --         inclock <='0';
           game_col_address <= game_col_address + '1';
           If game_col_address = "111111" Then
           game_row_address <= game_row_address + '1';
-- Swap Memory Banks When Done 
           IF game_row_address = "111111" Then
           generation_count <= generation_count + '1';
           active_page <= NOT active_page;
           single_gen_compute <= '0'; 
           End if;
           End if;
           neighbor_count_state <= "0000";
           nearest_neighbor_count <= "0000";
	WHEN OTHERS =>
           neighbor_count_state <= "0000";
           nearest_neighbor_count <= "0000";
           we<='0';
           neighbor_row_adjust <= "000000"; 
           neighbor_col_adjust <= "000000";
           game_col_address <= "000000";
           game_row_address <= "000000";
  END CASE;
 end if;
end if;
end if;
end process LIFE_GAME_COMPUTE;

-- Combine Flex Dip Switch Inputs into Switch vector
Switch <= Flex_Switch_8 & Flex_Switch_7 & Flex_Switch_6 & Flex_Switch_5 &
          Flex_Switch_4 & Flex_Switch_3 & Flex_Switch_2 & Flex_Switch_1;

-- Controls Speed of Game Updates with input from Flex Switches 7..4
SPEED_GAME: process (Clock)
begin
  wait until (Clock'Event) and (Clock='1');
if Speed = Switch_Sync(7 Downto 4) then
 Speed <="0000"; 
 Speed_Signal <='1';
ELSE
 Speed <= Speed + 1;
 Speed_Signal <='0';
End if;
end process SPEED_GAME;

-- Makes PB1 the Game Pause Button
PAUSE_GAME: process (PB1_DEBOUNCED_SYNC)
begin
  wait until (PB1_DEBOUNCED_SYNC'Event) and (PB1_DEBOUNCED_SYNC='1');
    Game_Run <= Not Game_Run;
End process PAUSE_GAME;



--Generate Horizontal and Vertical Timing Signals for Video Signal
VIDEO_DISPLAY: Process
Begin

Wait until(Clock'Event) and (Clock='1');
If Reset = '1' Then
 H_count <= CONV_STD_LOGIC_VECTOR(0,10);
 V_count <= CONV_STD_LOGIC_VECTOR(0,10);
 Video_on_H <= '0';
 Video_on_V <= '0';
Else
-- H_count counts pixels (640 + extra time for sync signals)
--
--   <-Clock out RGB Pixel Row Data ->   <-H Sync->
--   ------------------------------------__________--------
--   0                           640   659       755    799
--
If (H_count >= H_max) then
   H_count <= To_Stdlogicvector(B"0000000000");
Else
   H_count <= H_count + To_Stdlogicvector(B"0000000001");
End if;

--Generate Horizontal Sync Signal
If (H_count <= CONV_STD_LOGIC_VECTOR(755,10)) and (H_count >= CONV_STD_LOGIC_VECTOR(659,10)) Then
   Horiz_Sync <= '0';
ELSE
   Horiz_Sync <= '1';
End if;

--V_count counts rows of pixels (480 + extra time for sync signals)
--
--  <---- 480 Horizontal Syncs (pixel rows) -->  ->V Sync<-
--  -----------------------------------------------_______------------
--  0                                       480    493-494          524
--
If (V_count >= V_max) and (H_count >= CONV_STD_LOGIC_VECTOR(699,10)) then
   V_count <= To_Stdlogicvector(B"0000000000");
Else If (H_count = CONV_STD_LOGIC_VECTOR(699,10)) Then
   V_count <= V_count + To_Stdlogicvector(B"0000000001");
End if;
End if;

-- Generate Vertical Sync Signal
If (V_count <= CONV_STD_LOGIC_VECTOR(494,10)) and (V_count >= CONV_STD_LOGIC_VECTOR(493,10)) Then
   Vert_Sync <= '0';
   Debounce_CLock <= '0';
ELSE
   Vert_Sync <= '1';
   Debounce_Clock <= '1';
End if;

--Generate Mouse Clock Using Horizontal Sync Signal for Slow Clock (Save Counter Hardware)
If (H_count <= CONV_STD_LOGIC_VECTOR(400,10)) Then
   Buf_Mouse_CLK  <= '0';
ELSE
   Buf_Mouse_CLK  <= '1';
End if;


-- Generate Video on Screen Signals for Pixel Data
-- Generate row and col address for 10 by 8 superpixel to map into 64 by 64 video memory
--
If (H_count <= CONV_STD_LOGIC_VECTOR(639,10)) Then
   video_on_H <= '1';
If pixel_col_count < CONV_STD_LOGIC_VECTOR(9,6) Then
   pixel_col_count <= pixel_col_count + '1';
Else
   pixel_col_count <= "000000";
   col_address <= col_address + '1';
End if;
ELSE
   video_on_H <= '0';
   pixel_col_count <= "000000";
   col_address <= "000000";
End if;

IF(H_COUNT = CONV_STD_LOGIC_VECTOR(641,10)) Then
    pixel_row_count <= pixel_row_count + '1';
If (pixel_row_count = CONV_STD_LOGIC_VECTOR(7,6)) THEN
   pixel_row_count <= "000000";
   row_address <= row_address + '1';
End if;
End if;

If (V_count <= CONV_STD_LOGIC_VECTOR(479,10)) Then
   video_on_V <= '1';
ELSE
   video_on_V <= '0';
   pixel_row_count <= "000000";
   row_address <= "000000";
End if;


If (V_count = CONV_STD_LOGIC_VECTOR(0,10)) and (H_count = CONV_STD_LOGIC_VECTOR(0,10)) then
If (F_count = CONV_STD_LOGIC_VECTOR(30,5)) then
   F_count <= To_Stdlogicvector(B"00000");
--   Color_count <= Color_count + To_Stdlogicvector(B"0001");
Else 
   F_count <= F_count + To_Stdlogicvector(B"00001");
End if;
End if;
End if;

-- Generate RGB Pixel Data for Mouse Cursor
--If (H_Count(8 Downto 5) = Y_CURSOR(15 Downto 12)) and 
--(V_Count(8 Downto 5) = X_CURSOR(15 Downto 12)) Then
--Red_Data <= NOT Color_count(2);
--Green_Data<= NOT Color_count(1);
--Blue_Data<= NOT Color_count(0);
--Else
--Red_Data <= Color_count(2);
--Green_Data <= Color_count(1);
--Blue_Data <= Color_count(0);
--End if;

end process VIDEO_DISPLAY;

-- Connections to LED Display Segments
--LSB_dp <= LEFT_BUTTON;
    --LEFT MOUSE BUTTON;
--MSB_dp <= RIGHT_BUTTON;
    --RIGHT MOUSE BUTTON;
--LSB_dp <=  NOT MOUSE_CLK;
--MSB_dp <=  NOT MOUSE_DATA;
MSB_dp <= PB1_SYNC;
LSB_dp <= PB1_DEBOUNCED;
--LSB <=Y_CURSOR(15 Downto 12);
--MSB <=X_CURSOR(15 Downto 12);
MSB <= GENERATION_COUNT(7 Downto 4);
LSB <= GENERATION_COUNT(3 Downto 0);

MSB_a <= NOT MSB_7SEG(6);
MSB_b <= NOT MSB_7SEG(5);
MSB_c <= NOT MSB_7SEG(4);
MSB_d <= NOT MSB_7SEG(3);
MSB_e <= NOT MSB_7SEG(2);
MSB_f <= NOT MSB_7SEG(1);
MSB_g <= NOT MSB_7SEG(0);

LSB_a <= NOT LSB_7SEG(6);
LSB_b <= NOT LSB_7SEG(5);
LSB_c <= NOT LSB_7SEG(4);
LSB_d <= NOT LSB_7SEG(3);
LSB_e <= NOT LSB_7SEG(2);
LSB_f <= NOT LSB_7SEG(1);
LSB_g <= NOT LSB_7SEG(0);

LED_DISPLAY: process  (MSB,LSB)
-- BCD to 7 Segment Decoders for LED Displays
begin
CASE MSB IS
        WHEN "0000" =>
            MSB_7SEG <= "1111110";
        WHEN "0001" =>
            MSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            MSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            MSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            MSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            MSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            MSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            MSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            MSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            MSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            MSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            MSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            MSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            MSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            MSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            MSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            MSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

CASE LSB IS
        WHEN "0000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111110");
        WHEN "0001" =>
            LSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            LSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            LSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            LSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            LSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            LSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            LSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            LSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            LSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            LSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            LSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            LSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            LSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            LSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            LSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

end process LED_DISPLAY;

-- mouse connections
RESET <= Switch_Sync(1);
DELTA_Y <= "00000000" & PACKET_CHAR3 WHEN (PACKET_CHAR1(5) XOR PACKET_CHAR1(7)) ='0' 
ELSE (NOT("00000000" & PACKET_CHAR3) + '1');
DELTA_X <= "00000000" & PACKET_CHAR2 WHEN (PACKET_CHAR1(4) XOR PACKET_CHAR1(6)) ='0' 
ELSE (NOT("00000000" & PACKET_CHAR2) + '1');


--This process reads in serial data coming from the PS2 Mouse Port
-- This process is not working now and is under development
RECV_UART: process(MOUSE_CLK, MOUSE_DATA, RESET)
begin
If RESET='1' then
	INCNT <= conv_std_logic_vector(0,4);
    DATA_READY<= '0';
    READ_CHAR <= '0';
    PACKET_COUNT <= conv_std_logic_vector(0,2);
    X_CURSOR <= "10000000000000000";
    Y_CURSOR <= "10000000000000000";
    LEFT_BUTTON <= '0';
    RIGHT_BUTTON <= '0';
else
IF (MOUSE_CLK'event) and (MOUSE_CLK='1') Then 
--  Process Mouse Packet and update Mouse Cursor Postition
  IF PACKET_COUNT = "11" THEN
    PACKET_COUNT <= "00";
    DATA_READY<='0';
    Y_CURSOR <= Y_CURSOR + DELTA_Y;
    X_CURSOR <= X_CURSOR + DELTA_X;
    LEFT_BUTTON <= PACKET_CHAR1(0);
    RIGHT_BUTTON <= PACKET_CHAR1(1);


  else
-- Wait for RTS - Mouse Data Available
-- If MOUSE_DATA='1' AND DATA_READY='0' then
--    DATA_READY<='1';
-- else
-- Wait for Start Bit
 If MOUSE_DATA='0' AND READ_CHAR='0' then
	READ_CHAR<= '1';
 else
-- Reading in a Character SHIFT IN NEXT SERIAL BIT
  If READ_CHAR = '1' then
	IF INCNT/= "1001" then
         INCNT <= INCNT + conv_std_logic_vector(1,4);
	 SHIFTIN(7 DOWNTO 0) <= SHIFTIN(8 DOWNTO 1);
	 SHIFTIN(8) <= Mouse_DATA;
-- END OF CHARACTER save data
	 else
	 CHARIN <= SHIFTIN(7 DOWNTO 0);
     READ_CHAR<='0';
	 INCNT <= conv_std_logic_vector(0,4);
-- CHECK FOR FIRST CHARACTER OF PACKET TO SYNC PACKET
--     IF ((PACKET_COUNT = "00") AND (SHIFTIN(2) = '0')) 
--		OR PACKET_COUNT /= "00" Then 
     PACKET_COUNT <= PACKET_COUNT + conv_std_logic_vector(1,2);
--     end if;
-- FIFO for three character Mouse Packet
     PACKET_CHAR1 <= PACKET_CHAR2;    
     PACKET_CHAR2 <= PACKET_CHAR3;
     PACKET_CHAR3 <= SHIFTIN(7 DOWNTO 0);
 
	end if;
  end if;
 end if;
end if;
end if;
end if;
--end if;
end process RECV_UART;

--MOUSE_CLK <= 'Z';
--GEN_MOUSE_CLK: process
--begin
--  wait until (clock'event) and (clock='1');
--	MOUSE_CLK <= BUF_MOUSE_CLK OR NOT DATA_READY;
--end process GEN_MOUSE_CLK;

-- Sync extenal pushbutton inputs to chip clock
PUSH_BUTTON: process (clock)
begin
  wait until (clock'event) and (clock='1');
PB1_Sync <= NOT PB1;
PB2_Sync <= NOT PB2;
Switch_Sync <= Switch;
PB1_DEBOUNCED_SYNC <= PB1_DEBOUNCED;
PB2_DEBOUNCED_SYNC <= PB2_DEBOUNCED;
end process PUSH_BUTTON;

-- Debounce Button: Filters out mechanical bounce for around 80Ms.
-- Debounce clock uses Vert_Sync timing signal (16Ms) to save hardware
-- for clock prescaler
DEBOUNCE_BUTTON1: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB1(2 Downto 0) <= SHIFT_PB1(3 Downto 1);
  SHIFT_PB1(3) <= PB1_Sync;
  If SHIFT_PB1(3 Downto 0)="1111" THEN
    PB1_DEBOUNCED <= '1';
  ELSE 
    PB1_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON1;

DEBOUNCE_BUTTON2: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB2(2 Downto 0) <= SHIFT_PB2(3 Downto 1);
-- use switch 0 since PB2 broken on my board
  SHIFT_PB2(3) <= Switch_Sync(0);
  If SHIFT_PB2(3 Downto 0)="1111" THEN
    PB2_DEBOUNCED <= '1';
  ELSE 
    PB2_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON2;

SINGLE_PULSE_PB1: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
  If RESET='1' Then
  PB1_SINGLE_PULSE <='0';
  PB1_DEBOUNCED_DELAY <= '1';
  ELSE
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
  IF PB1_DEBOUNCED_SYNC = '1' AND PB1_DEBOUNCED_DELAY = '0' THEN
   PB1_SINGLE_PULSE <= '1';
  ELSE
   PB1_SINGLE_PULSE <= '0';
  END IF;
  PB1_DEBOUNCED_DELAY <= PB1_DEBOUNCED_SYNC;
 End if;
end process SINGLE_PULSE_PB1;

TEMP_PB1_SINGLE_PULSE <= PB1_SINGLE_PULSE;

SINGLE_PULSE_PB2: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
   If RESET='1' Then
    PB2_SINGLE_PULSE <='0';
    PB2_DEBOUNCED_DELAY <= '1';
   ELSE
    PB2_DEBOUNCED_DELAY <= PB2_DEBOUNCED_SYNC;
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
    IF PB2_DEBOUNCED_SYNC = '1' AND PB2_DEBOUNCED_DELAY = '0' THEN
     PB2_SINGLE_PULSE <= '1';
    ELSE
     PB2_SINGLE_PULSE <= '0';
    END IF;
  End if;
end process SINGLE_PULSE_PB2;


end behavior;

